<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaBaseExt\Controller;

use TYPO3\CMS\Core\Resource\File;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
use TYPO3\CMS\Extbase\Mvc\Web\Request;
use TYPO3\CMS\Extbase\Mvc\Web\Response;
use TYPO3\CMS\Extbase\Mvc\Web\Routing\UriBuilder;
use TYPO3\CMS\Extbase\Object\ObjectManagerInterface;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;
use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;

/**
 * @package     imia_base_ext
 * @subpackage  Controller
 * @author      David Frerich <d.frerich@imia.de>
 */
abstract class ActionController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
    /**
     * @var array
     */
    protected $fileStorage = array();

    /**
     * @var array
     */
    protected $hookObjects;

    protected function initializeView($view)
    {
        parent::initializeView($view);

        if ($this->getSetting('templateRootPath')) {
            $view->setTemplateRootPath($this->getSetting('templateRootPath'));
        }
    }

    /**
     * @return Request
     */
    public function getRequest()
    {
        return $this->request;
    }

    /**
     * @return Response
     */
    public function getResponse()
    {
        return $this->response;
    }

    /**
     * @return ObjectManagerInterface
     */
    public function getObjectManager()
    {
        return $this->objectManager;
    }

    /**
     * @return UriBuilder
     */
    public function getUriBuilder()
    {
        return $this->uriBuilder;
    }

    /**
     * @param string $key
     * @return mixed
     */
    public function getSetting($key)
    {
        if (strpos($key, '.') === false) {
            $value = $this->settings[$key];
        } else {
            $keys = explode('.', $key);

            $value = $this->settings[array_shift($keys)];
            foreach ($keys as $subKey) {
                if ($value && is_array($value)) {
                    $value = $value[$subKey];
                } else {
                    $value = null;
                    break;
                }
            }
        }

        if (is_array($value)) {
            return $this->renderTSObj($value);
        } else {
            return $value;
        }
    }

    /**
     * @param array $config
     * @return mixed
     */
    public function renderTSObj(array $config)
    {
        if (array_key_exists('_typoScriptNodeValue', $config) && is_object($this->getContentObject())) {
            $contentType = $config['_typoScriptNodeValue'];
            unset($config['_typoScriptNodeValue']);
            $contentConfig = $this->objectManager->get('TYPO3\CMS\Extbase\Service\TypoScriptService')
                ->convertPlainArrayToTypoScriptArray($config);

            return $this->getContentObject()->cObjGetSingle($contentType, $contentConfig);
        } else {
            return $config;
        }
    }

    /**
     * @return ContentObjectRenderer
     */
    public function getContentObject()
    {
        return $this->configurationManager->getContentObject();
    }

    /**
     * Translate by key
     *
     * @param string $key
     * @param array $arguments
     * @return string
     */
    public function translate($key, $arguments = array())
    {
        return (string)LocalizationUtility::translate($key, $this->extensionName, $arguments);
    }

    /**
     * @param array $recipient
     * @param array $sender
     * @param string $subject
     * @param string $body
     * @param string $format
     * @param function $closure
     * @return boolean
     */
    public function sendMail(array $recipient, array $sender, $subject, $body, $format = 'plain', $closure = null)
    {
        $message = $this->objectManager->get('TYPO3\CMS\Core\Mail\MailMessage');

        switch ($format) {
            case 'html':
                break;
            case 'plain':
            default:
                $format = 'plain';
                break;
        }



        foreach ($recipient as $key => $value) {
            if (!GeneralUtility::validEmail($key)) {
                if (!GeneralUtility::validEmail($value)) {
                    unset($recipient[$key]);
                }
            }
        }
        foreach ($sender as $key => $value) {
            if (!GeneralUtility::validEmail($key)) {
                if (!GeneralUtility::validEmail($value)) {
                    unset($sender[$key]);
                }
            }
        }

        if (count($recipient) > 0 && count($sender) > 0) {
            $message->setTo($recipient)
                ->setFrom($sender)
                ->setSubject($subject)
                ->setBody($body, 'text/' . $format);

            if ($closure) {
                try {
                    call_user_func($closure, $message);
                } catch(\Exception $e) {}
            }

            $message->send();

            return $message->isSent();
        } else {
            return false;
        }
    }

    /**
     * @param array $recipient
     * @param array $sender
     * @param string $subject
     * @param string $templateName
     * @param array $arguments
     * @param string $format
     * @param function $closure
     * @return boolean
     */
    public function sendTemplateMailSpecial(array $recipient, array $sender, $subject, $templateName, array $arguments = array(),
        $format = 'plain', $closure = null)
    {
        $subject = $this->renderFluidContent($subject, $arguments);

        return $this->sendTemplateMail($recipient, $sender, $subject, $templateName, $arguments, $format, $closure);
    }

    /**
     * @param array $recipient
     * @param array $sender
     * @param string $subject
     * @param string $templateName
     * @param array $arguments
     * @param string $format
     * @param function $closure
     * @return boolean
     */
    public function sendTemplateMail(array $recipient, array $sender, $subject, $templateName, array $arguments = array(),
        $format = 'plain', $closure = null)
    {
        switch ($format) {
            case 'html':
                $ext = 'html';
                break;
            case 'text':
            case 'plain':
            default:
                $ext = 'txt';
                break;
        }

        $message = $this->renderView('Email', $templateName, $arguments, $ext);

        return $this->sendMail(
            $recipient,
            $sender,
            $subject,
            $message,
            $format,
            $closure
        );
    }

    /**
     * @param string $name
     * @return array
     */
    public function getFileArgument($name)
    {
        $fileArgument = null;

        $argumentName = explode('.', $name);
        $name = array_shift($argumentName);

        if ($this->request->hasArgument($name) && array_key_exists($this->getPluginNamespace(), $_FILES)
          && $_FILES[$this->getPluginNamespace()]['name'][$name]) {
            $fileArgument = array(
                'name'      => $_FILES[$this->getPluginNamespace()]['name'][$name],
                'type'      => $_FILES[$this->getPluginNamespace()]['type'][$name],
                'tmp_name'  => $_FILES[$this->getPluginNamespace()]['tmp_name'][$name],
                'error'     => $_FILES[$this->getPluginNamespace()]['error'][$name],
                'size'      => $_FILES[$this->getPluginNamespace()]['size'][$name],
            );

            if (count($argumentName) > 0) {
                foreach ($argumentName as $subname) {
                    foreach ($fileArgument as $key => $argument) {
                        $fileArgument[$key] = $argument[$subname];
                    }
                }
            }
        }

        return $fileArgument;
    }

    /**
     * @param array $fileData
     * @param integer $storageUid
     * @param string $folder
     * @return File
     */
    public function uploadFile(array $fileData, $storageUid = 0, $folder = '')
    {
        $file = null;
        $storage = $this->getFileStorage($storageUid);

        if ($storage->isWritable() && file_exists($fileData['tmp_name'])) {
            $targetFolder = $storage->getRootLevelFolder();
            foreach (explode('/', $folder) as $folderName) {
                if ($folderName) {
                    if ($storage->hasFolderInFolder($folderName, $targetFolder)) {
                        $targetFolder = $targetFolder->getSubfolder($folderName);
                    } else {
                        $targetFolder = $targetFolder->createFolder($folderName);
                    }
                }
            }

            $file = $targetFolder->addUploadedFile($fileData, 'changeName');
            if ($file) {
                // indexes file
                $file->updateProperties(array());
            }
        }

        return $file;
    }

    /**
     * @param string $filePath
     * @param string $fileName
     * @param integer $storageUid
     * @param string $folder
     * @return File
     */
    public function addFile($filePath, $fileName, $storageUid = 0, $folder = '')
    {
        $file = null;
        $storage = $this->getFileStorage($storageUid);

        if ($storage->isWritable() && file_exists($filePath)) {
            $targetFolder = $storage->getRootLevelFolder();
            foreach (explode('/', $folder) as $folderName) {
                if ($folderName) {
                    if ($storage->hasFolderInFolder($folderName, $targetFolder)) {
                        $targetFolder = $targetFolder->getSubfolder($folderName);
                    } else {
                        $targetFolder = $targetFolder->createFolder($folderName);
                    }
                }
            }

            $file = $targetFolder->addFile($filePath, $fileName, 'changeName');
            if ($file) {
                // indexes file
                $file->updateProperties(array());
            }
        }

        return $file;
    }

    /**
     * @param File $file
     * @param integer $storageUid
     * @param string $folder
     * @return File
     */
    public function moveFile(File $file, $storageUid = 0, $folder = '')
    {
        $storage = $this->getFileStorage($storageUid);
        if ($storage->isWritable()) {
            $targetFolder = $storage->getRootLevelFolder();
            foreach (explode('/', $folder) as $folderName) {
                if ($folderName) {
                    if ($storage->hasFolderInFolder($folderName, $targetFolder)) {
                        $targetFolder = $targetFolder->getSubfolder($folderName);
                    } else {
                        $targetFolder = $targetFolder->createFolder($folderName);
                    }
                }
            }

            $file->moveTo($targetFolder);
        }

        return $file;
    }

    /**
     * @param string $fileName
     * @param integer $storageUid
     * @param string $folder
     * @return File
     */
    public function getFile($fileName, $storageUid = 0, $folder = '')
    {
        $file = null;
        $storage = $this->getFileStorage($storageUid);
        if ($storage->isWritable()) {
            $fileFolder = $storage->getRootLevelFolder();
            foreach (explode('/', $folder) as $folderName) {
                if ($folderName) {
                    if ($storage->hasFolderInFolder($folderName, $fileFolder)) {
                        $fileFolder = $fileFolder->getSubfolder($folderName);
                    } else {
                        $fileFolder = $fileFolder->createFolder($folderName);
                    }
                }
            }

            if ($storage->hasFile($fileFolder->getIdentifier() . $fileName)) {
                $file = $storage->getFile($fileFolder->getIdentifier() . $fileName);
            }
        }

        return $file;
    }

    /**
     * @param integer $storageUid
     * @return \TYPO3\CMS\Core\Resource\ResourceStorage
     */
    public function getFileStorage($storageUid = 0)
    {
        if (!array_key_exists($storageUid, $this->fileStorage)) {
            $factory = $this->objectManager->get('TYPO3\CMS\Core\Resource\ResourceFactory');
            $this->fileStorage[$storageUid] = $factory->getStorageObject($storageUid);
        }

        return $this->fileStorage[$storageUid];
    }

    /**
     * @param string $content
     * @param array $arguments
     * @return string
     */
    public function renderFluidContent($content, $arguments = array())
    {
        $tempPath = PATH_site . 'typo3temp/baseext_' . md5($content) . '_' .
            substr(sha1(bin2hex(mcrypt_create_iv(22, MCRYPT_DEV_URANDOM))), 0, 10) . '.temp';

        file_put_contents($tempPath, $content);
        $rendered = $this->renderTemplate($tempPath, $arguments);
        unlink($tempPath);

        return $rendered;
    }

    /**
     * @param string $controller
     * @param string $action
     * @param array $arguments
     * @param string $format
     * @return string
     */
    public function renderView($controller, $action, $arguments, $ext = 'html')
    {
        $templateRootPaths = $this->getPaths('templateRootPath');
        do {
            $templateRootPath = array_pop($templateRootPaths);
            $templatePathAndFilename = $templateRootPath . $controller . '/' . $action . '.' . $ext;
        } while (!file_exists($templatePathAndFilename) && count($templateRootPaths) > 0);

        return $this->renderTemplate($templatePathAndFilename, $arguments);
    }

    /**
     * @param string $controller
     * @param string $action
     * @param array $arguments
     * @param string $format
     * @return string
     */
    public function renderTemplate($templatePathAndFilename, $arguments, $ext = 'html')
    {
        $view = $this->objectManager->get('TYPO3\CMS\Fluid\View\StandaloneView');
        $view->setFormat($ext);
        $view->getRequest()->setControllerExtensionName($this->extensionName);

        $partialRootPaths = $this->getPaths('partialRootPath');
        if (method_exists($view, 'setPartialRootPaths')) {
            $view->setPartialRootPaths($partialRootPaths);
        } else {
            $view->setPartialRootPath($partialRootPaths[0]);
        }
        $layoutRootPaths = $this->getPaths('layoutRootPath');
        if (method_exists($view, 'setLayoutRootPaths')) {
            $view->setLayoutRootPaths($layoutRootPaths);
        } else {
            $view->setLayoutRootPath($layoutRootPaths[0]);
        }

        $view->setTemplatePathAndFilename($templatePathAndFilename);
        if ($arguments['settings'] && is_array($arguments['settings'])) {
            if ($this->settings && is_array($this->settings)) {
                $arguments['settings'] = GeneralUtility::array_merge_recursive_overrule($this->settings, $arguments['settings']);
            }
        } else {
            $arguments['settings'] = $this->settings;
        }

        $view->assignMultiple($arguments);

        return $view->render();
    }

    /**
     * Function: sanitize
     * Returns a sanitized string, typically for URLs.
     *
     * Parameters:
     *     $string - The string to sanitize.
     *     $forceLowercase - Force the string to lowercase?
     *     $anal - If set to *true*, will remove all non-alphanumeric characters.
     */
    public function sanitize($string, $forceLowercase = true, $anal = false)
    {
        $strip = array("~", "`", "!", "@", "#", "$", "%", "^", "&", "*", "(", ")", "_", "=", "+", "[", "{", "]",
            "}", "\\", "|", ";", ":", "\"", "'", "&#8216;", "&#8217;", "&#8220;", "&#8221;", "&#8211;", "&#8212;",
            ",", "<", ".", ">", "/", "?");

        $clean = trim(str_replace($strip, "", strip_tags($string)));
        $clean = preg_replace('/\s+/', "-", $clean);
        $clean = ($anal) ? preg_replace("/[^a-zA-Z0-9]/", "", $clean) : $clean ;

        return ($forceLowercase) ?
            (function_exists('mb_strtolower')) ?
                mb_strtolower($clean, 'UTF-8') :
                strtolower($clean) :
            $clean;
    }

    /**
     * @return array
     */
    public function getExtensionConfiguration()
    {
        return $this->configurationManager->getConfiguration(
            ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);
    }

    /**
     * @return array
     */
    public function getSettings()
    {
        if ($this->settings && is_array($this->settings)) {
            foreach ($this->settings as $key => $setting) {
                $this->settings[$key] = $this->getSetting($key);
            }
        }

        return $this->settings;
    }

    /**
     * @param string $name
     * @return array
     */
    protected function getPaths($name)
    {
        $extensionConfiguration = $this->getExtensionConfiguration();

        $paths = array();
        if (is_array($extensionConfiguration['view']) && array_key_exists('templateRootPaths', $extensionConfiguration['view'])
            && is_array($extensionConfiguration['view'][$name . 's']) && count($extensionConfiguration['view'][$name . 's']) > 0) {
            ksort($extensionConfiguration['view'][$name . 's']);
            foreach ($extensionConfiguration['view'][$name . 's'] as $path) {
                $paths[] = GeneralUtility::getFileAbsFileName($path);
            }
        } else {
            if ($extensionConfiguration['view'][$name]) {
                $paths[] = GeneralUtility::getFileAbsFileName($extensionConfiguration['view'][$name]);
            } else {
                switch ($name) {
                    case 'partialRootPath':
                        $defaultPath = 'Resources/Private/Partials/';
                        break;
                    case 'layoutRootPath':
                        $defaultPath = 'Resources/Private/Layouts/';
                        break;
                    case 'templateRootPath':
                    default:
                        $defaultPath = 'Resources/Private/Templates/';
                        break;
                }

                $paths[] = ExtensionManagementUtility::extPath(
                    GeneralUtility::camelCaseToLowerCaseUnderscored($this->extensionName),
                    $defaultPath
                );
            }
        }

        return $paths;
    }

    /**
     * @param string $method
     * @param array $args
     */
    protected function callHook($method, $args)
    {
        foreach ($this->getHookObjects() as $hookObject) {
            if (method_exists($hookObject, $method)) {
                call_user_func_array(array($hookObject, $method), $args);
            }
        }
    }

    /**
     * @return array
     */
    protected function getHookObjects()
    {
        if (!$this->hookObjects) {
            $this->hookObjects = array();
            $className = get_class($this);
            $className = substr($className, strrpos($className, '\\') + 1);

            $extKey = 'tx_' . strtolower($this->extensionName);
            $controllerName = $this->request->getControllerName() ?: ($this->controllerName ?: str_replace('Controller', '', $className));
            if (is_array($GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][$extKey][$controllerName])) {
                foreach ($GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][$extKey][$controllerName] as $_classRef) {
                    $this->hookObjects[] = GeneralUtility::getUserObj($_classRef);
                }
            }
        }

        return $this->hookObjects;
    }

    /**
     * Load arguments for a different plugin than the current
     *
     * @param string $pluginName
     */
    protected function loadArguments($pluginName = null)
    {
        if ($pluginName) {
            $arguments = GeneralUtility::_GPmerged($this->getPluginNamespace($pluginName));
        } else {
            $arguments = GeneralUtility::array_merge_recursive_overrule($_GET, $_POST);
        }

        $this->request->setArguments($arguments);
    }

    /**
     *
     * @param string $pluginName
     * @param string $extensionName
     * @return string
     */
    protected function getPluginNamespace($pluginName = null, $extensionName = null)
    {
        $extensionService = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Service\\ExtensionService');
        $pluginNamespace = $extensionService->getPluginNamespace(
            $extensionName ? $extensionName : $this->extensionName,
            $pluginName ? $pluginName : $this->request->getPluginName()
        );

        return $pluginNamespace;
    }
}